import slackweb

from pywell.entry_points import run_from_cli


DESCRIPTION = 'Send notification to Slack.'

ARG_DEFINITIONS = {
    'SLACK_WEBHOOK': 'Web hook URL for Slack.',
    'SLACK_CHANNEL': 'Slack channel to send to.',
    'SLACK_MESSAGE_TEXT': 'Text to send.'
}

REQUIRED_ARGS = [
    'SLACK_WEBHOOK', 'SLACK_CHANNEL', 'SLACK_MESSAGE_TEXT'
]


def notify_slack(args):
    slack = slackweb.Slack(url=args.SLACK_WEBHOOK)
    return slack.notify(text=args.SLACK_MESSAGE_TEXT, channel=args.SLACK_CHANNEL)


if __name__ == '__main__':
    run_from_cli(notify_slack, DESCRIPTION, ARG_DEFINITIONS, REQUIRED_ARGS)



from setuptools import setup

setup(
    name='mo_pywell',
    version="1.0.0",
    author='Scott Reynen',
    packages=['pywell'],
    url='https://github.com/MoveOnOrg/pywell',
    install_requires=[
        "boto3",
        "botocore"
    ],
    license='MIT',
    description="A collection of independent Python scripts following the Unix philosphy of doing one thing well. Forked from https://github.com/sreynen/pywell.",
)



import psycopg2
import psycopg2.extras

from pywell.entry_points import run_from_cli
from pywell.secrets_manager import get_secret


DESCRIPTION = 'Get results from a database query.'

ARG_DEFINITIONS = {
    'DB_HOST': 'Database host IP or hostname.',
    'DB_PORT': 'Database port number.',
    'DB_USER': 'Database user.',
    'DB_PASS': 'Database password.',
    'DB_NAME': 'Database name.',
    'DB_QUERY': 'Database query, as string or file path ending with ".sql".',
    'NO_RESULTS': 'Optional True flag if the query just needs to run without results.',
    'DB_VALUES': 'Optional values to fill placeholders in query.'
}

REQUIRED_ARGS = [
    'DB_HOST', 'DB_PORT', 'DB_USER', 'DB_PASS', 'DB_NAME', 'DB_QUERY'
]


def get_psql_results(args):
    database = psycopg2.connect(
        host=args.DB_HOST,
        port=args.DB_PORT,
        user=args.DB_USER,
        password=args.DB_PASS,
        database=args.DB_NAME
    )
    database_cursor = database.cursor(
            cursor_factory=psycopg2.extras.RealDictCursor
    )
    if args.DB_QUERY[-4:] == '.sql':
        with open(args.DB_QUERY, 'r') as file:
            query = file.read()
    else:
        query = args.DB_QUERY
    if 'DB_VALUES' in args.__dict__ and args.DB_VALUES:
        database_cursor.execute(query, args.DB_VALUES)
    else:
        database_cursor.execute(query)
    if 'NO_RESULTS' in args.__dict__ and args.NO_RESULTS:
        database.commit()
        return []
    return [dict(row) for row in database_cursor.fetchall()]


# REQUIRED_ARGS = [ 'DB_QUERY' ]
# OPTIONAL_ARGS = [ 'NO_RESULTS', 'DB_VALUES' ]
def get_psql_results_remote_creds(args, db_secret_name):

    db_creds = get_secret(db_secret_name)

    database = psycopg2.connect(
        host = db_creds['host'],
        port = db_creds['port'],
        user = db_creds['username'],
        password = db_creds['password'],
        database = db_creds['dbName']
    )
    database_cursor = database.cursor(
            cursor_factory=psycopg2.extras.RealDictCursor
    )
    if args.DB_QUERY[-4:] == '.sql':
        with open(args.DB_QUERY, 'r') as file:
            query = file.read()
    else:
        query = args.DB_QUERY
    if 'DB_VALUES' in args.__dict__ and args.DB_VALUES:
        database_cursor.execute(query, args.DB_VALUES)
    else:
        database_cursor.execute(query)
    if 'NO_RESULTS' in args.__dict__ and args.NO_RESULTS:
        database.commit()
        return []
    return [dict(row) for row in database_cursor.fetchall()]


if __name__ == '__main__':
    run_from_cli(get_psql_results, DESCRIPTION, ARG_DEFINITIONS, REQUIRED_ARGS)



from pywell.secrets_manager import get_secret
import datetime

def get_settings(definitions, secret_name) -> dict:
    """
    Get settings from AWS Secrets Manager or settings.py file, return as dictionary.
    """
    import os

    if secret_name:
        return get_secret(secret_name)

    if os.path.exists(os.path.join(os.getcwd(), 'settings.py')):
        import settings as settings_file
        settings = {}
        for argname in definitions:
            settings[argname] = getattr(settings_file, argname, False)
        return settings
    
    return {}

def all_required_args_set(args, required, definitions) -> bool:
    """
    Check that all requried args are set, print details on any missing.
    """
    set = True

    for arg in required:
        if not getattr(args, arg, False):
            print('%s (%s) required, missing.' % (definitions.get(arg), arg))
            set = False

    return set

def run_from_cli(func, description, definitions, required, secret_name='') -> None:
    """
    Entry point via command line.
    """
    import argparse
    import pprint

    settings = get_settings(definitions, secret_name)

    parser = argparse.ArgumentParser(description=description)

    for argname, helptext in definitions.items():
        parser.add_argument(
            '--%s' % argname, dest=argname, help=helptext,
            default=settings.get(argname, False)
        )

    args = parser.parse_args()

    if all_required_args_set(args, required, definitions):
        pprint.PrettyPrinter(indent=2).pprint(func(args))

class Struct:
    def __init__(self, **entries):
        self.__dict__.update(entries)

def json_serial(obj):
    """
    JSON serializer for objects not serializable by default JSON code.
    """
    if isinstance(obj, datetime.datetime):
        return obj.isoformat()
    raise TypeError("Type %s not serializable." % type(obj))

def run_from_lambda(func, description, definitions, required, event, secret_name='') -> str:
    """
    Entry point from Amazon Lambda.
    """
    import json

    settings = get_settings(definitions, secret_name)

    kwargs = event.get('kwargs', False)
    if kwargs:
        for argname in kwargs.keys():
            event[argname] = kwargs.get(argname)

    for argname in definitions:
        if not event.get(argname, False):
            event[argname] = settings.get(argname, False)

    args = Struct(**event)

    if all_required_args_set(args, required, definitions):
        return json.dumps(func(args), default=json_serial)


def run_from_api_gateway(func, description, definitions, required, event, format='JSON', filename='', secret_name='') -> str:
    """
    Entry point from Amazon Lambda via API Gateway.
    """
    import json
    from urllib.parse import parse_qsl

    settings = get_settings(definitions, secret_name)

    kwargs = event.get('kwargs', False)
    if kwargs:
        for argname in kwargs.keys():
            event[argname] = kwargs.get(argname)

    if event.get('httpMethod', '') == 'POST':
        try:
            post_body = json.loads(event.get('body', '{}'))
        except:
            try:
                post_body = dict(parse_qsl(event.get('body', '')))
            except:
                post_body = {}
        for argname in definitions:
            if not event.get(argname, False) and argname in post_body.keys():
                event[argname] = post_body.get(argname, False)

    for argname in definitions:
        if not event.get(argname, False):
            event[argname] = settings.get(argname, False)

    args = Struct(**event)

    if all_required_args_set(args, required, definitions):
        result = func(args)
        if format == 'CSV':
            import csv
            import io
            if len(result):
                headers = result[0].keys()
            else:
                headers = []
            output = io.StringIO()
            writer = csv.DictWriter(
                output,
                quoting=csv.QUOTE_NONNUMERIC,
                fieldnames=headers
            )
            writer.writeheader()
            for data in result:
                writer.writerow(data)
            return {
                'isBase64Encoded': True,
                'statusCode': 200,
                'headers': {
                    'Content-Type': 'text/csv',
                    'Content-Disposition': 'attachment;filename=%s' % filename
                },
                "body": output.getvalue()
            }
        else:
            return {
                'isBase64Encoded': False,
                'statusCode': 200,
                'headers': {'Content-Type': 'application/json'},
                "body": json.dumps(result)
            }



from botocore.exceptions import ClientError
import base64
import boto3
import json
import logging

def get_secret(secret_name, region_name='us-west-1'):
    logging.getLogger().setLevel('INFO')
    # Create a Secrets Manager client
    session = boto3.session.Session(region_name=region_name)
    client = session.client(
        service_name='secretsmanager'
    )

    get_secret_value_response = client.get_secret_value(
        SecretId=secret_name
    )

    # Decrypts secret using the associated KMS CMK. Depending on whether
    # the secret is a string or binary, one of these fields will be
    # populated.
    if 'SecretString' in get_secret_value_response:
        secret = get_secret_value_response['SecretString']
    else:
        secret = base64.b64decode(
            get_secret_value_response['SecretBinary']
        )

    return json.loads(secret)
